﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using DocumentFormat.OpenXml.Packaging;
using OpenXmlPowerTools;

class Program
{
    static void Main(string[] args)
    {
        var fileToFix = new FileInfo("../../../Failed STDElement Doc - 3.docx");
        var newWmlDoc = DocumentFixer(new WmlDocument(fileToFix.FullName));
        var fi = new FileInfo("../../../Failed STDElement Doc - 3b.docx");
        if (fi.Exists)
            fi.Delete();
        newWmlDoc.SaveAs(fi.FullName);
    }

    static WmlDocument DocumentFixer(WmlDocument wmlDoc)
    {
        using (OpenXmlMemoryStreamDocument streamDoc = new OpenXmlMemoryStreamDocument(wmlDoc))
        {
            using (WordprocessingDocument wDoc = streamDoc.GetWordprocessingDocument())
            {
                foreach (var part in wDoc.ContentParts())
                {
                    var runsWithMultipleRpr = part
                        .GetXDocument()
                        .Descendants(W.r)
                        .Where(r => r.Elements(W.rPr).Count() > 1)
                        .ToList();
                    foreach (var run in runsWithMultipleRpr)
                    {
                        run.Elements(W.rPr).Skip(1).Remove();
                    }
                    part.PutXDocument();
                }
                foreach (var altChunkPart in wDoc.MainDocumentPart.AlternativeFormatImportParts)
                {
                    WmlDocument fixedDoc;
                    using (Stream partStream = altChunkPart.GetStream())
                    {
                        byte[] ba = ReadFully(partStream);
                        var doc = new WmlDocument("file.docx", ba);
                        fixedDoc = DocumentFixer(doc);
                    }
                    using (MemoryStream ms = new MemoryStream())
                    {
                        ms.Write(fixedDoc.DocumentByteArray, 0, fixedDoc.DocumentByteArray.Length);
                        ms.Seek(0, SeekOrigin.Begin);
                        altChunkPart.FeedData(ms);
                    }
                }
            }
            return streamDoc.GetModifiedWmlDocument();
        }
    }

    public static byte[] ReadFully(Stream input)
    {
        byte[] buffer = new byte[16 * 1024];
        using (MemoryStream ms = new MemoryStream())
        {
            int read;
            while ((read = input.Read(buffer, 0, buffer.Length)) > 0)
            {
                ms.Write(buffer, 0, read);
            }
            return ms.ToArray();
        }
    }
}